@@ -61,7 +61,7 @@
 #include <ctype.h> // isspace()
 
 
-#define CHECK_MK_VERSION "1.1.10"
+#define CHECK_MK_VERSION "1.1.12"
 #define CHECK_MK_AGENT_PORT 6556
 #define SERVICE_NAME "Check_MK_Agent"
 #define KiloByte 1024
@@ -103,6 +103,16 @@ struct ipspec g_only_from[MAX_ONLY_FROM];
 unsigned int g_num_only_from = 0;
 
 
+struct winperf_counter {
+    int   id;
+    char *name;
+};
+
+#define MAX_WINPERF_COUNTERS 64
+struct winperf_counter g_winperf_counters[MAX_WINPERF_COUNTERS];
+unsigned int g_num_winperf_counters = 0;
+
+
 #ifdef DEBUG
 void debug(char *text)
 {
@@ -135,7 +145,7 @@ void verbose(const char *format, ...)
 
 void outputCounter(SOCKET &out, BYTE *datablock, int counter, 
 		   PERF_OBJECT_TYPE *objectPtr, PERF_COUNTER_DEFINITION *counterPtr);
-void outputCounterValue(SOCKET &out, PERF_COUNTER_BLOCK *counterBlockPtr, int size, unsigned offset);
+void outputCounterValue(SOCKET &out, PERF_COUNTER_DEFINITION *counterPtr, PERF_COUNTER_BLOCK *counterBlockPtr);
 double current_time();
 
 
@@ -298,6 +308,44 @@ void section_ps(SOCKET &out)
 }
 
 
+// Determine the start type of a service. Unbelievable how much
+// code is needed for that...
+const char *service_start_type(SC_HANDLE scm, LPCTSTR service_name)
+{
+    // Query the start type of the service
+    const char *start_type = "invalid1";
+    SC_HANDLE schService;
+    LPQUERY_SERVICE_CONFIG lpsc; 
+    schService = OpenService(scm, service_name, SERVICE_QUERY_CONFIG);
+    if (schService) {
+        start_type = "invalid2";
+        DWORD dwBytesNeeded, cbBufSize;
+        if (!QueryServiceConfig(schService, NULL, 0, &dwBytesNeeded)) {
+            start_type = "invalid3";
+            DWORD dwError = GetLastError();
+            if (dwError == ERROR_INSUFFICIENT_BUFFER) {
+                start_type = "invalid4";
+                cbBufSize = dwBytesNeeded;
+                lpsc = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LMEM_FIXED, cbBufSize);
+                if (QueryServiceConfig(schService, lpsc, cbBufSize, &dwBytesNeeded)) {
+                    switch (lpsc->dwStartType) {
+                    case SERVICE_AUTO_START:    start_type = "auto"; break; 
+                    case SERVICE_BOOT_START:    start_type = "boot"; break;   
+                    case SERVICE_DEMAND_START:  start_type = "demand"; break;
+                    case SERVICE_DISABLED:      start_type = "disabled"; break;   
+                    case SERVICE_SYSTEM_START:  start_type = "system"; break;
+                    default:                    start_type = "other";
+                    }
+                }
+                LocalFree(lpsc);
+            }
+        }
+        CloseServiceHandle(schService); 
+    }
+    return start_type;
+}
+
+
 void section_services(SOCKET &out)
 {
     output(out, "<<<services>>>\n");
@@ -328,6 +376,8 @@ void section_services(SOCKET &out)
 			case SERVICE_STOP_PENDING:     state_name = "stopping"; break;
 			case SERVICE_STOPPED:          state_name = "stopped"; break;
 			}
+                        
+                        const char *start_type = service_start_type(scm, service->lpServiceName);
 
 			// The service name usually does not contain spaces. But
 			// in some cases it does. We replace them with _ in order
@@ -338,8 +388,8 @@ void section_services(SOCKET &out)
 				*w = '_';
 			}			
 			
-			output(out, "%-18s %-8s %s\n", 
-			       service->lpServiceName, state_name,
+			output(out, "%s %s/%s %s\n", 
+			       service->lpServiceName, state_name, start_type,
 			       service->lpDisplayName);
 			service ++;
 		    }
@@ -377,11 +427,14 @@ PERF_INSTANCE_DEFINITION *NextInstance (PERF_INSTANCE_DEFINITION *pInstance) {
 }
 
 
-void dump_performance_counters(SOCKET &out, unsigned counter)
+void dump_performance_counters(SOCKET &out, unsigned counter_base_number, const char *countername)
 {
+    output(out, "<<<winperf_%s>>>\n", countername);
+    output(out, "%.2f %u\n", current_time(), counter_base_number);
+
     // registry entry is ascii representation of counter index
     char counter_index_name[8];
-    snprintf(counter_index_name, sizeof(counter_index_name), "%u", counter);
+    snprintf(counter_index_name, sizeof(counter_index_name), "%u", counter_base_number);
 
     // allocate block to store counter data block
     DWORD size = DEFAULT_BUFFER_SIZE;
@@ -418,12 +471,12 @@ void dump_performance_counters(SOCKET &out, unsigned counter)
   
     // Now walk through the list of objects. The bad news is:
     // even if we expect only one object, windows might send
-    // us more that one object. We need to scan a list of objects
+    // us more than one object. We need to scan a list of objects
     // in order to find the one we have asked for. >:-P
     for (unsigned int a=0 ; a < dataBlockPtr->NumObjectTypes ; a++) 
     {
 	// Have we found the object we seek? 
-	if (objectPtr->ObjectNameTitleIndex == counter)
+	if (objectPtr->ObjectNameTitleIndex == counter_base_number)
 	{
 	    // Yes. Great. Now: each object consist of a lot of counters.
 	    // We walk through the list of counters in this object:
@@ -435,15 +488,36 @@ void dump_performance_counters(SOCKET &out, unsigned counter)
 	    // to find the beginning of the data block (which comes after the
 	    // counter definitions)
 	    PERF_COUNTER_DEFINITION *last_counter = FirstCounter(objectPtr);
-	    for(unsigned int b=0 ; b < objectPtr->NumCounters ; b++) 
+	    for (unsigned int b=0 ; b < objectPtr->NumCounters ; b++) 
 		last_counter = NextCounter(last_counter);
 	    BYTE *datablock = (BYTE *)last_counter;
+            
+            // In case of multi-instance objects, output a list of all instance names
+            int num_instances = objectPtr->NumInstances;
+            if (num_instances >= 0) 
+            {
+                output(out, "%d instances:", num_instances);
+                char name[512];
+                PERF_INSTANCE_DEFINITION *instancePtr = FirstInstance(objectPtr);
+                for(int b=0 ; b<objectPtr->NumInstances ; b++) 
+                {
+                    WCHAR *name_start = (WCHAR *)((char *)(instancePtr) + instancePtr->NameOffset);
+                    memcpy(name, name_start, instancePtr->NameLength);
+                    WideCharToMultiByte(CP_UTF8, 0, name_start, instancePtr->NameLength, name, sizeof(name), NULL, NULL);
+                    // replace spaces with '_'
+                    for (char *s = name; *s; s++) 
+                        if (*s == ' ') *s = '_';
 
-	    // Now walk through the counter list a second time and output
-	    // all non-zero counters
-	    for(unsigned int b=0 ; b < objectPtr->NumCounters ; b++) 
+                    output(out, " %s", name);
+	            instancePtr = NextInstance(instancePtr);
+                }
+                output(out, "\n");
+            }
+
+	    // Now walk through the counter list a second time and output all counters
+	    for (unsigned int b=0 ; b < objectPtr->NumCounters ; b++) 
 	    {
-		outputCounter(out, datablock, counter, objectPtr, counterPtr);
+		outputCounter(out, datablock, counter_base_number, objectPtr, counterPtr);
 		counterPtr = NextCounter(counterPtr);
 	    }
 	}
@@ -454,91 +528,93 @@ void dump_performance_counters(SOCKET &out, unsigned counter)
 }
 
 
-void outputCounter(SOCKET &out, BYTE *datablock, int counter, 
+void outputCounter(SOCKET &out, BYTE *datablock, int counter_base_number, 
 		   PERF_OBJECT_TYPE *objectPtr, PERF_COUNTER_DEFINITION *counterPtr)
 {
-    int num_instances = objectPtr->NumInstances;
-    unsigned offset = counterPtr->CounterOffset;
-    int size = counterPtr->CounterSize;
 
     // determine the type of the counter (for verbose output)
-    const char *countertypename = "(unknown)";
+    const char *countertypename = 0;
     switch (counterPtr->CounterType) {
     case PERF_COUNTER_COUNTER:                countertypename = "counter"; break;
-    case PERF_COUNTER_QUEUELEN_TYPE:          countertypename = "queuelen_type" ; break;
-    case PERF_SAMPLE_COUNTER:                 countertypename = "sample_counter"; break;
-//  case PERF_OBJ_TIME_TIMER:                 countertypename = "obj_time_timer"; break;
-//  case PERF_COUNTER_100NS_QUEUELEN_TYPE:    countertypename = "100ns_queuelen_type"; break;
-//  case PERF_COUNTER_OBJ_TIME_QUEUELEN_TYPE: countertypename = "obj_time_queuelen_type"; break;
     case PERF_COUNTER_TIMER:                  countertypename = "timer"; break;
-    case PERF_COUNTER_TIMER_INV:              countertypename = "timer_inv"; break;
+    case PERF_COUNTER_QUEUELEN_TYPE:          countertypename = "queuelen_type"; break;
     case PERF_COUNTER_BULK_COUNT:             countertypename = "bulk_count"; break;
-//  case PERF_COUNTER_LARGE_QUEUELEN_TYPE:    countertypename = "large_queuelen_type"; break;
-    case PERF_COUNTER_MULTI_TIMER:            countertypename = "multi_timer"; break;
-    case PERF_COUNTER_MULTI_TIMER_INV:        countertypename = "multi_timer_inv"; break;
+    case PERF_COUNTER_TEXT:                   countertypename = "text"; break;
     case PERF_COUNTER_RAWCOUNT:               countertypename = "rawcount"; break;
-    case PERF_COUNTER_RAWCOUNT_HEX:           countertypename = "rawcount_hex"; break;
-//  case PERF_COUNTER_DELTA:                  countertypename = "delta"; break;
     case PERF_COUNTER_LARGE_RAWCOUNT:         countertypename = "large_rawcount"; break;
-    case PERF_COUNTER_LARGE_RAWCOUNT_HEX:     countertypename = "large_rawcount_hex"; break;
-//  case PERF_COUNTER_LARGE_DELTA:            countertypename = "large_delta"; break;
+    case PERF_COUNTER_RAWCOUNT_HEX:           countertypename = "rawcount_hex"; break;
+    case PERF_COUNTER_LARGE_RAWCOUNT_HEX:     countertypename = "large_rawcount_HEX"; break;
+    case PERF_SAMPLE_FRACTION:                countertypename = "sample_fraction"; break;
+    case PERF_SAMPLE_COUNTER:                 countertypename = "sample_counter"; break;
+    case PERF_COUNTER_NODATA:                 countertypename = "nodata"; break;
+    case PERF_COUNTER_TIMER_INV:              countertypename = "timer_inv"; break;
+    case PERF_SAMPLE_BASE:                    countertypename = "sample_base"; break;
+    case PERF_AVERAGE_TIMER:                  countertypename = "average_timer"; break;
+    case PERF_AVERAGE_BASE:                   countertypename = "average_base"; break;
+    case PERF_AVERAGE_BULK:                   countertypename = "average_bulk"; break;
     case PERF_100NSEC_TIMER:                  countertypename = "100nsec_timer"; break;
     case PERF_100NSEC_TIMER_INV:              countertypename = "100nsec_timer_inv"; break;
+    case PERF_COUNTER_MULTI_TIMER:            countertypename = "multi_timer"; break;
+    case PERF_COUNTER_MULTI_TIMER_INV:        countertypename = "multi_timer_inV"; break;
+    case PERF_COUNTER_MULTI_BASE:             countertypename = "multi_base"; break;
     case PERF_100NSEC_MULTI_TIMER:            countertypename = "100nsec_multi_timer"; break;
-    case PERF_100NSEC_MULTI_TIMER_INV:        countertypename = "100nsec_multi_timer_inv"; break;
-    case PERF_SAMPLE_FRACTION:                countertypename = "sample_fraction"; break;
+    case PERF_100NSEC_MULTI_TIMER_INV:        countertypename = "100nsec_multi_timer_inV"; break;
     case PERF_RAW_FRACTION:                   countertypename = "raw_fraction"; break;
-//  case PERF_LARGE_RAW_FRACTION:             countertypename = "large_raw_fraction"; break;
-//  case PERF_PRECISION_SYSTEM_TIMER:         countertypename = "precision_system_timer"; break;
-//  case PERF_PRECISION_100NS_TIMER:          countertypename = "precision_100ns_timer"; break;
-//  case PERF_PRECISION_OBJECT_TIMER:         countertypename = "precision_object_timer"; break;
-    case PERF_AVERAGE_TIMER:                  countertypename = "average_timer"; break;
-    case PERF_AVERAGE_BULK:                   countertypename = "average_bulk"; break;
-    case PERF_SAMPLE_BASE:        	      countertypename = "sample_base"; break;
-    case PERF_AVERAGE_BASE:       	      countertypename = "average_base"; break;
-    case PERF_COUNTER_MULTI_BASE: 	      countertypename = "multi_base"; break;
-    case PERF_RAW_BASE:           	      countertypename = "raw_base"; break;
-//  case PERF_LARGE_RAW_BASE:     	      countertypename = "large_raw_base"; break;
-    case PERF_ELAPSED_TIME:       	      countertypename = "elapsed_time"; break;
-    case PERF_COUNTER_TEXT:       	      countertypename = "text"; break;
-    case PERF_COUNTER_NODATA:     	      countertypename = "nodata"; break;
-    case PERF_COUNTER_HISTOGRAM_TYPE:         countertypename = "histogram_type"; break;
+    case PERF_RAW_BASE:                       countertypename = "raw_base"; break;
+    case PERF_ELAPSED_TIME:                   countertypename = "elapsed_time"; break;
     }
 	      
     // Output index of counter object and counter, and timestamp
-    output(out, "%d:%d %.2f", counter, counterPtr->CounterNameTitleIndex, current_time());
+    output(out, "%d", counterPtr->CounterNameTitleIndex - counter_base_number);
 
     // If this is a multi-instance-counter, loop over the instances
+    int num_instances = objectPtr->NumInstances;
     if (num_instances >= 0) 
     {
 	// get pointer to first instance
 	PERF_INSTANCE_DEFINITION *instancePtr = FirstInstance(objectPtr);
 		  
-	for(int b=0 ; b<objectPtr->NumInstances ; b++) 
+	for (int b=0 ; b<objectPtr->NumInstances ; b++) 
 	{
 	    // PERF_COUNTER_BLOCK dieser Instanz ermitteln.
 	    PERF_COUNTER_BLOCK *counterBlockPtr = GetCounterBlock(instancePtr);
-	    outputCounterValue(out, counterBlockPtr, size, offset);
+	    outputCounterValue(out, counterPtr, counterBlockPtr);
 	    instancePtr = NextInstance(instancePtr);
 	}
 
     }
     else { // instanceless counter
 	PERF_COUNTER_BLOCK *counterBlockPtr = (PERF_COUNTER_BLOCK *) datablock;
-	outputCounterValue(out, counterBlockPtr, size, offset);
+	outputCounterValue(out, counterPtr, counterBlockPtr);
     }
-    output(out, " %s\n", countertypename);
+    if (countertypename)
+        output(out, " %s\n", countertypename);
+    else
+        output(out, " type(%lx)\n", counterPtr->CounterType);
 }
 
 
-void outputCounterValue(SOCKET &out, PERF_COUNTER_BLOCK *counterBlockPtr, int size, unsigned offset)
+void outputCounterValue(SOCKET &out, PERF_COUNTER_DEFINITION *counterPtr, PERF_COUNTER_BLOCK *counterBlockPtr)
 {
-    if (size == 4) {
-	DWORD value = *((DWORD *)((BYTE *)counterBlockPtr) + offset);
+    unsigned offset = counterPtr->CounterOffset;
+    int size = counterPtr->CounterSize;
+    BYTE *pData = ((BYTE *)counterBlockPtr) + offset;
+
+    if (counterPtr->CounterType | PERF_SIZE_DWORD) 
+	output(out, " %llu", (ULONGLONG)(*(DWORD*)pData));
+
+    else if (counterPtr->CounterType | PERF_SIZE_LARGE)
+	output(out, " %llu", *(UNALIGNED ULONGLONG*)pData);
+   
+    // handle other data generically. This is wrong in some situation.
+    // Once upon a time in future we might implement a conversion as
+    // described in http://msdn.microsoft.com/en-us/library/aa373178%28v=vs.85%29.aspx
+    else if (size == 4) {
+	DWORD value = *((DWORD *)pData);
 	output(out, " %lu", value);
     }
     else if (size == 8) {
-	DWORD *data_at = (DWORD *)(((BYTE *)counterBlockPtr) + offset);
+	DWORD *data_at = (DWORD *)pData;
 	DWORDLONG value = (DWORDLONG)*data_at + ((DWORDLONG)*(data_at + 1) << 32);
 	output(out, " %s", llu_to_string(value));
     }
@@ -881,15 +957,12 @@ void section_mem(SOCKET &out)
 
 void section_winperf(SOCKET &out)
 {
+    dump_performance_counters(out, 234, "phydisk");
+    dump_performance_counters(out, 238, "processor");
 
-    output(out, "<<<winperf>>>\n");
-  
-    for (int i=0; i<700; i+=2) {
-	if (i != 230 && i != 232 && i != 786 && i != 740)
-	    dump_performance_counters(out, i);
-    }
-    // Terminalservices
-    dump_performance_counters(out, 2102);
+    // also output additionally configured counters
+    for (unsigned i=0; i<g_num_winperf_counters; i++)
+        dump_performance_counters(out, g_winperf_counters[i].id, g_winperf_counters[i].name);
 }
 
 
@@ -995,15 +1068,26 @@ void section_eventlog(SOCKET &out)
 
 char *add_interpreter(char *path, char *newpath)
 {
-    if (!strcmp(path + strlen(path) - 5, ".vbs\"")) {
+    if (!strcmp(path + strlen(path) - 4, ".vbs")) {
         // If this is a vbscript don't rely on the default handler for this
         // file extensions. This might be notepad or some other editor by
         // default on a lot of systems. So better add cscript as interpreter.
-        snprintf(newpath, 256, "cscript.exe //Nologo %s", path);
+        snprintf(newpath, 256, "cscript.exe //Nologo \"%s\"", path);
+        return newpath;
+    }
+    else if (!strcmp(path + strlen(path) - 4, ".ps1")) {
+        // Same for the powershell scripts. Add the powershell interpreter.
+        // To make this work properly two things are needed:
+        //   1.) The powershell interpreter needs to be in PATH
+        //   2.) The execution policy needs to allow the script execution
+        //       -> Get-ExecutionPolicy / Set-ExecutionPolicy
+        snprintf(newpath, 256, "powershell.exe -NoLogo -ExecutionPolicy RemoteSigned \"& \'%s\'\"", path);
+        return newpath;
+    }
+    else {
+        snprintf(newpath, 256, "\"%s\"", path);
         return newpath;
     }
-    else
-        return path;
 }
 
 void run_plugin(SOCKET &out, char *path) 
@@ -1030,7 +1114,7 @@ void run_external_programs(SOCKET &out, char *dirname)
         while (0 != (de = readdir(dir))) {
             char *name = de->d_name;
             if (name[0] != '.') {
-                snprintf(path, sizeof(path), "\"%s\\%s\"", dirname, name);
+                snprintf(path, sizeof(path), "%s\\%s", dirname, name);
                 run_plugin(out, path);
             }
         }
@@ -1066,8 +1150,14 @@ void get_agent_dir(char *buffer, int size)
             while (end > buffer && *end != '\\')
                 end--;
             *end = 0; // replace \ with string end => get directory of executable
+
+            // Handle case where name is quoted with double quotes. 
+            // This is reported to happen on some 64 Bit systems when spaces
+            // are in the directory name.
+            if (*buffer == '"') {
+                memmove(buffer, buffer + 1, strlen(buffer));
+            }
         }
-        size = dsize;
         RegCloseKey(key);
     }
     else {
@@ -1184,7 +1274,7 @@ void listen_tcp_loop()
 
     SOCKET connection;
     // Loop for ever.
-    debug("Die Schleife gestartet.");
+    debug("Starting main loop.");
     while (!should_terminate) 
     {
 	// Das Dreckswindows kann nicht vernuenftig gleichzeitig auf
@@ -1409,11 +1499,10 @@ void do_debug()
 {
     verbose_mode = true;
     do_tcp = false;
-    logwatch_send_initial_entries = true;
-    logwatch_suppress_info = false;
+    // logwatch_send_initial_entries = true;
+    // logwatch_suppress_info = false;
     SOCKET dummy;
-    find_eventlogs(dummy);
-    //section_eventlog(dummy);
+    output_data(dummy);
 }
 
 void do_adhoc()
@@ -1527,31 +1616,70 @@ void add_only_from(char *value)
     g_num_only_from ++;
 }
 
-void parse_only_from(char *value)
+char *next_word(char **line)
 {
-    char *end = value + strlen(value);
+    char *end = *line + strlen(*line);
+    char *value = *line;
     while (value < end) {
         value = lstrip(value);
         char *s = value;
         while (*s && !isspace(*s))
             s++;
         *s = 0;
-        add_only_from(value);
-        value = s + 1;
+        *line = s + 1;
+        rstrip(value);
+        if (strlen(value) > 0)
+            return value;
+        else
+            return 0;
     }
+    return 0;
 }
 
-void handle_config_variable(char *var, char *value)
+
+void parse_only_from(char *value)
+{
+    char *word;
+    while (0 != (word = next_word(&value)))
+        add_only_from(word);
+}
+
+void handle_global_config_variable(char *var, char *value)
 {
     if (!strcmp(var, "only_from")) {
         parse_only_from(value);
     }
     else {
-        fprintf(stderr, "Invalid configuration variable %s.\r\n", var);
+        fprintf(stderr, "Invalid configuration variable %s in section [global].\r\n", var);
         exit(1);
     }
 }
 
+void handle_winperf_config_variable(char *var, char *value)
+{
+    if (!strcmp(var, "counters")) {
+        char *word;
+        while (0 != (word = next_word(&value))) {
+            if (g_num_winperf_counters >= MAX_WINPERF_COUNTERS) {
+                fprintf(stderr, "Defined too many counters in [winperf]:counters.\r\n");
+                exit(1);
+            }
+            char *colon = strchr(word, ':');
+            if (!colon) {
+                fprintf(stderr, "Invalid counter '%s' in section [winperf]: need number and colon, e.g. 238:processor.\n", word);
+                exit(1);
+            }
+            *colon = 0;
+            g_winperf_counters[g_num_winperf_counters].name = strdup(colon + 1);
+            g_winperf_counters[g_num_winperf_counters].id = atoi(word);
+            g_num_winperf_counters ++;
+        }
+    }
+    else {
+        fprintf(stderr, "Invalid configuration variable %s in section [winperf].\r\n", var);
+        exit(1);
+    }
+}
 
 void read_config_file()
 {
@@ -1564,6 +1692,8 @@ void read_config_file()
 
     char line[512];
     int lineno = 0;
+    void (*variable_handler)(char *var, char *value) = 0;
+
     while (!feof(file)) {
         if (!fgets(line, sizeof(line), file))
             return;
@@ -1576,12 +1706,20 @@ void read_config_file()
             // found section header
             l[len-1] = 0;
             char *section = l + 1;
-            if (strcmp(section, "global")) {
-                fprintf(stderr, "Invalid section %s in %s.\r\n",
-                        section, g_config_file);
+            if (!strcmp(section, "global"))
+                variable_handler = handle_global_config_variable;
+            else if (!strcmp(section, "winperf"))
+                variable_handler = handle_winperf_config_variable;
+            else {
+                fprintf(stderr, "Invalid section [%s] in %s in line %d.\r\n",
+                        section, g_config_file, lineno);
                 exit(1);
             }
         }
+        else if (!variable_handler) {
+            fprintf(stderr, "Line %d is outside of any section.\r\n", lineno);
+            exit(1);
+        }
         else {
             // split up line at = sign
             char *s = l;
@@ -1597,7 +1735,7 @@ void read_config_file()
             char *variable = l;
             rstrip(variable);
             value = strip(value);
-            handle_config_variable(variable, value); // add section later
+            variable_handler(variable, value); // add section later
         }
     }
         
