@@ -24,7 +24,7 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
-import socket, os, sys, time, re, signal, math
+import socket, os, sys, time, re, signal, math, tempfile
 
 # Python 2.3 does not have 'set' in normal namespace.
 # But it can be imported from 'sets'
@@ -33,7 +33,6 @@ try:
 except NameError:
     from sets import Set as set
 
-
 # colored output, if stdout is a tty
 if sys.stdout.isatty():
     tty_red       = '\033[31m'
@@ -86,6 +85,8 @@ g_hostname                   = "unknown" # Host currently being checked
 g_aggregated_service_results = {}   # store results for later submission
 compiled_regexes             = {}   # avoid recompiling regexes
 nagios_command_pipe          = None # Filedescriptor to open nagios command pipe.
+checkresult_file_fd          = None
+checkresult_file_path        = None
 g_single_oid_hostname        = None
 g_single_oid_cache           = {}
 g_broken_snmp_hosts          = set([])
@@ -193,10 +194,8 @@ def submit_aggregated_results(hostname):
             else:
                 text = " *** ".join([ item + " " + output for itemstatus, item, output in outputlist ])
 
-        if not opt_dont_submit and nagios_command_pipe:
-            nagios_command_pipe.write("[%d] PROCESS_SERVICE_CHECK_RESULT;%s;%s;%d;%s\n" %
-                (int(time.time()), aggr_hostname, servicedesc, status, text))
-            nagios_command_pipe.flush()
+        if not opt_dont_submit:
+            submit_to_nagios(aggr_hostname, servicedesc, status, text)
 
         if opt_verbose:
             color = { 0: tty_green, 1: tty_yellow, 2: tty_red, 3: tty_magenta }[status]
@@ -214,16 +213,15 @@ def submit_check_mk_aggregation(hostname, status, output):
         return
 
     if not opt_dont_submit:
-        open_command_pipe()
-        if nagios_command_pipe:
-            nagios_command_pipe.write("[%d] PROCESS_SERVICE_CHECK_RESULT;%s;Check_MK;%d;%s\n" %
-                  (int(time.time()), summary_hostname(hostname), status, output))
-            nagios_command_pipe.flush()
+        submit_to_nagios(summary_hostname(hostname), "Check_MK", status, output)
+
     if opt_verbose:
         color = { 0: tty_green, 1: tty_yellow, 2: tty_red, 3: tty_magenta }[status]
         print "%-20s %s%s%-70s%s" % ("Check_MK", tty_bold, color, output, tty_normal)
 
 
+
+
 #   +----------------------------------------------------------------------+
 #   |                 ____      _         _       _                        |
 #   |                / ___| ___| |_    __| | __ _| |_ __ _                 |
@@ -331,20 +329,6 @@ def get_realhost_info(hostname, ipaddress, checkname, max_cache_age):
         write_cache_file(cache_relpath, repr(table) + "\n")
         return table
 
-    # now try SNMP explicity values
-    try:
-        mib, baseoid, suffixes = snmp_info_single[checkname]
-    except:
-        baseoid = None
-    if baseoid:
-        content = read_cache_file(cache_relpath, max_cache_age)
-        if content:
-            return eval(content)
-        table = get_snmp_explicit(hostname, ipaddress, mib, baseoid, suffixes)
-        store_cached_checkinfo(hostname, checkname, table)
-        write_cache_file(cache_relpath, repr(table) + "\n")
-        return table
-
     # No SNMP check. Then we must contact the check_mk_agent. Have we already
     # to get data from the agent? If yes we must not do that again! Even if
     # no cache file is present
@@ -377,10 +361,10 @@ def read_cache_file(relpath, max_cache_age):
             result = f.read(10000000)
             f.close()
             if len(result) > 0:
-                if opt_verbose:
+                if opt_debug:
                     sys.stderr.write("Using data from cachefile %s.\n" % cachefile)
                 return result
-        elif opt_verbose:
+        elif opt_debug:
             sys.stderr.write("Skipping cache file %s: Too old\n" % cachefile)
 
     if simulation_mode and not opt_no_cache:
@@ -413,25 +397,26 @@ def write_cache_file(relpath, output):
 # In that case it will be looked up if needed. Also caching will
 # be handled here
 def get_agent_info(hostname, ipaddress, max_cache_age):
-    result = read_cache_file(hostname, max_cache_age)
-    if result:
-        return result
+    output = read_cache_file(hostname, max_cache_age)
+    if not output:
+        # Try to contact every host only once
+        if hostname in g_broken_agent_hosts:
+            raise MKAgentError("")
 
-    # Try to contact every host only once
-    if hostname in g_broken_agent_hosts:
-	raise MKAgentError("")
+        # If the host ist listed in datasource_programs the data from
+        # that host is retrieved by calling an external program (such
+        # as ssh or rsy) instead of a TCP connect.
+        commandline = get_datasource_program(hostname, ipaddress)
+        if commandline:
+            output = get_agent_info_program(commandline)
+        else:
+            output = get_agent_info_tcp(hostname, ipaddress)
 
-    # If the host ist listed in datasource_programs the data from
-    # that host is retrieved by calling an external program (such
-    # as ssh or rsy) instead of a TCP connect.
-    commandline = get_datasource_program(hostname, ipaddress)
-    if commandline:
-        output = get_agent_info_program(commandline)
-    else:
-        output = get_agent_info_tcp(hostname, ipaddress)
+        # Got new data? Write to cache file
+        write_cache_file(hostname, output)
 
-    # Got new data? Write to cache file
-    write_cache_file(hostname, output)
+    if agent_simulator:
+        output = agent_simulator_process(output)
 
     return output
 
@@ -463,7 +448,7 @@ def get_agent_info_tcp(hostname, ipaddress):
             s.settimeout(tcp_connect_timeout)
         except:
             pass # some old Python versions lack settimeout(). Better ignore than fail
-        s.connect((ipaddress, agent_port))
+        s.connect((ipaddress, agent_port_of(hostname)))
         try:
             s.setblocking(1)
         except:
@@ -476,10 +461,15 @@ def get_agent_info_tcp(hostname, ipaddress):
             else:
                 break
         s.close()
+        if len(output) == 0: # may be caused by xinetd not allowing our address
+            raise MKAgentError("Empty output from agent at TCP port %d" % 
+                  agent_port_of(hostname))
         return output
+    except MKAgentError, e:
+        raise
     except Exception, e:
         raise MKAgentError("Cannot get data from TCP port %s:%d: %s" %
-                           (ipaddress, agent_port, e))
+                           (ipaddress, agent_port_of(hostname), e))
 
 
 # Gets all information about one host so far cached.
@@ -542,28 +532,6 @@ def parse_info(lines):
     return info
 
 
-def lookup_ipaddress(hostname):
-    if fake_dns:
-        return fake_dns
-
-    elif simulation_mode or opt_use_snmp_walk or \
-         (is_usewalk_host(hostname) and is_snmp_host(hostname)):
-        return "127.0.0.1"
-    elif hostname in g_dns_cache:
-        return g_dns_cache[hostname]
-    else:
-        ipa = ipaddresses.get(hostname)
-        if ipa:
-            return ipa
-        else:
-            try:
-                ipa = socket.gethostbyname(hostname)
-            except:
-                g_dns_cache[hostname] = None
-                raise
-            g_dns_cache[hostname] = ipa
-            return ipa
-
 def cachefile_age(filename):
     try:
         return time.time() - os.stat(filename)[8]
@@ -637,23 +605,42 @@ def get_counter(countername, this_time, this_val, allow_negative=False):
     return timedif, per_sec
 
 
+# Compute average by gliding exponential algorithm
+# itemname: unique id for storing this average until the next check
+# this_time: timestamp of new value
+# backlog: averaging horizon in minutes
+# initialize_zero: assume average of 0.0 when now previous average is stored
 def get_average(itemname, this_time, this_val, backlog, initialize_zero = True):
 
-    # first call: take current value as average
+    # first call: take current value as average or assume 0.0
     if not itemname in g_counters:
         if initialize_zero:
             this_val = 0
         g_counters[itemname] = (this_time, this_val)
-        return 1.0, this_val
+        return 1.0, this_val # avoid time diff of 0.0 -> avoid division by zero
 
     # Get previous value and time difference
     last_time, last_val = g_counters.get(itemname)
     timedif = this_time - last_time
 
-    # compute weight, but how?
-    weight = math.e ** (-timedif / backlog)
+    # Compute the weight: We do it like this: First we assume that
+    # we get one sample per minute. And that backlog is the number
+    # of minutes we should average over. Then we want that the weight
+    # of the values of the last average minutes have a fraction of W%
+    # in the result and the rest until infinity the rest (1-W%). 
+    # Then the weight can be computed as backlog'th root of 1-W
+    percentile = 0.50
+
+    weight_per_minute = (1 - percentile) ** (1.0 / backlog)
+
+    # now let's compute the weight per second. This is done 
+    weight = weight_per_minute ** (timedif / 60.0)
 
     new_val = last_val * weight + this_val * (1 - weight)
+
+    # print "Alt: %.5f, Jetzt: %.5f, Timedif: %.1f, Gewicht: %.5f, Neu: %.5f" % \
+    #     (last_val, this_val, timedif, weight, new_val)
+
     g_counters[itemname] = (this_time, new_val)
     return timedif, new_val
 
@@ -693,28 +680,31 @@ def do_check(hostname, ipaddress):
 
     try:
         load_counters(hostname)
-        agent_version, num_success, num_errors, problems = do_all_checks_on_host(hostname, ipaddress)
+        agent_version, num_success, error_sections, problems = do_all_checks_on_host(hostname, ipaddress)
+        num_errors = len(error_sections)
         save_counters(hostname)
         if problems:
-	    output = "CRIT - %s" % problems
+	    output = "CRIT - %s, " % problems
             status = 2
         elif num_errors > 0 and num_success > 0:
-            output = "WARNING - Got only %d out of %d infos" % (num_success, num_success + num_errors)
+            output = "WARN - Missing agent sections: %s - " % ", ".join(error_sections)
             status = 1
         elif num_errors > 0:
-            output = "CRIT - Got no information from host"
+            output = "CRIT - Got no information from host, "
             status = 2
         elif agent_min_version and agent_version < agent_min_version:
-            output = "WARNING - old plugin version %s (should be at least %s)" % (agent_version, agent_min_version)
+            output = "WARN - old plugin version %s (should be at least %s), " % (agent_version, agent_min_version)
             status = 1
         else:
-            output = "OK - Agent version %s" % agent_version
+            output = "OK - "
+            if agent_version != None:
+                output += "Agent version %s, " % agent_version
             status = 0
 
     except MKGeneralException, e:
         if opt_debug:
             raise
-        output = "UNKNOWN - %s" % e
+        output = "UNKNOWN - %s, " % e
         status = 3
 
     if aggregate_check_mk:
@@ -724,8 +714,11 @@ def do_check(hostname, ipaddress):
             if opt_debug:
                 raise
 
+    if checkresult_file_fd != None:
+        close_checkresult_file()
+
     run_time = time.time() - start_time
-    output += ", execution time %.1f sec|execution_time=%.3f\n" % (run_time, run_time)
+    output += "execution time %.1f sec|execution_time=%.3f\n" % (run_time, run_time)
     sys.stdout.write(output)
     sys.exit(status)
 
@@ -740,7 +733,7 @@ def do_all_checks_on_host(hostname, ipaddress):
     global g_hostname
     g_hostname = hostname
     num_success = 0
-    num_errors = 0
+    error_sections = set([])
     check_table = get_sorted_check_table(hostname)
     problems = []
 
@@ -758,14 +751,14 @@ def do_all_checks_on_host(hostname, ipaddress):
         except MKSNMPError, e:
 	    if str(e):
 	        problems.append(str(e))
-            num_errors += 1
+            error_sections.add(infotype)
 	    g_broken_snmp_hosts.add(hostname)
 	    continue
 
         except MKAgentError, e:
 	    if str(e):
                 problems.append(str(e))
-            num_errors += 1
+            error_sections.add(infotype)
 	    g_broken_agent_hosts.add(hostname)
 	    continue
 
@@ -815,7 +808,7 @@ def do_all_checks_on_host(hostname, ipaddress):
             if not dont_submit:
                 submit_check_result(hostname, description, result, aggrname)
         else:
-            num_errors += 1
+            error_sections.add(infotype)
 
     submit_aggregated_results(hostname)
 
@@ -824,17 +817,42 @@ def do_all_checks_on_host(hostname, ipaddress):
             version_info = get_host_info(hostname, ipaddress, 'check_mk')
             agent_version = version_info[0][1]
         else:
-            agent_version = "(unknown)"
+            agent_version = None
     except MKAgentError, e:
 	g_broken_agent_hosts.add(hostname)
         agent_version = "(unknown)"
     except:
         agent_version = "(unknown)"
-    return agent_version, num_success, num_errors, ", ".join(problems)
+    error_sections = list(error_sections)
+    error_sections.sort()
+    return agent_version, num_success, error_sections, ", ".join(problems)
+
+
+
+def open_checkresult_file():
+    global checkresult_file_fd
+    global checkresult_file_path
+    if checkresult_file_fd == None:
+        try:
+            checkresult_file_fd, checkresult_file_path = \
+                tempfile.mkstemp('', 'c', check_result_path)
+        except Exception, e:
+            raise MKGeneralException("Cannot create check result file in %s: %s" % 
+                    (check_result_path, e))
+
+
+def close_checkresult_file():
+    global checkresult_file_fd
+    if checkresult_file_fd != None:
+        os.close(checkresult_file_fd)
+        file(checkresult_file_path + ".ok", "w")
+        checkresult_file_fd = None
+
 
 def nagios_pipe_open_timeout(signum, stackframe):
     raise IOError("Timeout while opening pipe")
 
+
 def open_command_pipe():
     global nagios_command_pipe
     if nagios_command_pipe == None:
@@ -853,11 +871,21 @@ def open_command_pipe():
 
 
 
-def none_to_empty(x):
+def convert_perf_value(x):
     if x == None:
         return ""
-    else:
+    elif type(x) in [ str, unicode ]:
         return x
+    elif type(x) == float:
+        return ("%.6f" % x).rstrip("0").rstrip(".")
+    else:
+        return str(x)
+
+def convert_perf_data(p):
+    # replace None with "" and fill up to 7 values
+    p = (map(convert_perf_value, p) + ['','','',''])[0:6]
+    return "%s=%s;%s;%s;%s;%s" %  tuple(p)
+
 
 def submit_check_result(host, servicedesc, result, sa):
     global nagios_command_pipe
@@ -884,22 +912,15 @@ def submit_check_result(host, servicedesc, result, sa):
             check_command = None
 
         for p in perfdata:
-            # replace None with "" and fill up to 6 values
-            p = (map(none_to_empty, p) + ['','','',''])[0:6]
-            perftexts.append("%s=%s;%s;%s;%s;%s" %  tuple(p) )
-        if perftexts != [] and not direct_rrd_update(host, servicedesc, perfdata):
+            perftexts.append(convert_perf_data(p))
+
+        if perftexts != []:
             if check_command and perfdata_format == "pnp":
                 perftexts.append("[%s]" % check_command)
             perftext = "|" + (" ".join(perftexts))
 
     if not opt_dont_submit:
-        open_command_pipe()
-        if nagios_command_pipe:
-            nagios_command_pipe.write("[%d] PROCESS_SERVICE_CHECK_RESULT;%s;%s;%d;%s\n" %
-                                   (int(time.time()), host, servicedesc, result[0], result[1] + perftext)  )
-            # Important: Nagios needs the complete command in one single write() block!
-            # Python buffers and sends chunks of 4096 bytes, if we do not flush.
-            nagios_command_pipe.flush()
+        submit_to_nagios(host, servicedesc, result[0], result[1] + perftext)
 
     if opt_verbose:
         if opt_showperfdata:
@@ -909,21 +930,35 @@ def submit_check_result(host, servicedesc, result, sa):
         color = { 0: tty_green, 1: tty_yellow, 2: tty_red, 3: tty_magenta }[result[0]]
         print "%-20s %s%s%-56s%s%s" % (servicedesc, tty_bold, color, result[1], tty_normal, p)
 
-def direct_rrd_update(host, servicedesc, perfdata):
-    if do_rrd_update:
-        path = rrd_path + "/" + host.replace(":", "_") + "/" + servicedesc.replace("/", "_").replace(" ", "_")
-        # check existance and age of xml file
-        try:
-            xml_age = os.stat(path + ".xml").st_mtime
-            if time.time() - xml_age > (3600 + os.getpid() % 1800):
-                return False # XML file too old
-            numbers = [ str(p[1]).rstrip("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_ ") for p in perfdata ]
-            # sys.stderr.write("Schreibe rrdtool.update(" + path + ".rrd" + ", N:" + ":".join(numbers))
-            rrdtool.update(path + ".rrd", "N:" + ":".join(numbers))
-        except:
-            return False # Update not successfull or XML file missing or too old
-        return True
-    return False
+
+def submit_to_nagios(host, service, state, output):
+    if check_submission == "pipe":
+        open_command_pipe()
+        if nagios_command_pipe:
+            nagios_command_pipe.write("[%d] PROCESS_SERVICE_CHECK_RESULT;%s;%s;%d;%s\n" %
+                                   (int(time.time()), host, service, state, output)  )
+            # Important: Nagios needs the complete command in one single write() block!
+            # Python buffers and sends chunks of 4096 bytes, if we do not flush.
+            nagios_command_pipe.flush()
+    elif check_submission == "file":
+        open_checkresult_file()
+        if checkresult_file_fd:
+            now = time.time()
+            os.write(checkresult_file_fd, 
+                """host_name=%s
+service_description=%s
+check_type=1
+check_options=0
+reschedule_check
+latency=0.0
+start_time=%.1f
+finish_time=%.1f
+return_code=%d
+output=%s
+
+""" % (host, service, now, now, state, output))
+    else:
+        raise MKGeneralException("Invalid setting %r for check_submission. Must be 'pipe' or 'file'" % check_submission)
 
 
 #   +----------------------------------------------------------------------+
@@ -1026,7 +1061,11 @@ def get_nic_speed_human_readable(speed):
             speed = "100MBit/s"
         elif speedi == 1000000000:
             speed = "1GBit/s"
-        elif speed < 1500000000:
+        elif speed < 1500:
+            speed = "%dBit/s" % speedi
+        elif speed < 1000000:
+            speed = "%.1fKBit/s" % (speedi / 1000.0)
+        elif speed < 1000000000:
             speed = "%.2fMBit/s" % (speedi / 1000000.0)
         else:
             speed = "%.2fGBit/s" % (speedi / 1000000000.0)
