@@ -77,7 +77,7 @@ function registerEdgeListeners(obj) {
     else
         edges = [ obj ];
 
-    for(var i in edges) {
+    for(var i = 0; i < edges.length; i++) {
         // It is possible to open other domains in the content frame - don't register
         // the event in that case. It is not permitted by most browsers!
         if(!contentFrameAccessible())
@@ -117,10 +117,6 @@ function getButton(event) {
     return (event.which < 2) ? "LEFT" : ((event.which == 2) ? "MIDDLE" : "RIGHT");
 }
 
-function getTarget(event) {
-  return event.target ? event.target : event.srcElement;
-}
-
 function snapinStartDrag(event) {
   if (!event)
     event = window.event;
@@ -269,9 +265,11 @@ function snapinStopDrag(event) {
 
 function getDivChildNodes(node) {
   var children = [];
-  for(var i in node.childNodes)
-    if(node.childNodes[i].tagName === 'DIV')
-      children.push(node.childNodes[i]);
+  var childNodes = node.childNodes;
+  for(var i = 0; i < childNodes.length; i++)
+    if(childNodes[i].tagName === 'DIV')
+      children.push(childNodes[i]);
+  childNodes = null;
   return children;
 }
 
@@ -281,7 +279,7 @@ function getSnapinList() {
   
   var l = [];
   var childs = getDivChildNodes(snapinDragging.parentNode);
-  for(var i in childs) {
+  for(var i = 0; i < childs.length; i++) { 
     var child = childs[i];
     // Skip
     // - non snapin objects
@@ -322,7 +320,7 @@ function getSnapinTargetPos() {
 
   // Find the nearest snapin to current left/top corner of
   // the currently dragged snapin
-  for(var i in childs) {
+  for(var i = 0; i < childs.length; i++) { 
     var child = childs[i];
 
     if (!child.id || child.id.substr(0, 7) != 'snapin_' || child.id == snapinDragging.id)
@@ -375,20 +373,6 @@ function debug(s) {
   window.parent.frames[1].document.write(s+'<br />');
 }
 
-function pageHeight() {
-  var h;
-  
-  if (window.innerHeight !== null && typeof window.innerHeight !== 'undefined' && window.innerHeight !== 0)
-    h = window.innerHeight;
-  else if (document.documentElement && document.documentElement.clientHeight)
-    h = document.documentElement.clientHeight;
-  else if (document.body !== null)
-    h = document.body.clientHeight;
-  else
-    h = null;
-  
-  return h;
-}
 
 // Set the size of the sidebar_content div to fit the whole screen
 // but without scrolling. The height of the header and footer divs need
@@ -399,15 +383,15 @@ function setSidebarHeight() {
   var oFooter  = document.getElementById('side_footer');
   var height   = pageHeight();
 
-	// Don't handle zero heights
-	if(height == 0)
-		return;
+  // Don't handle zero heights
+  if (height == 0)
+    return;
 
   oContent.style.height = (height - oHeader.clientHeight - oFooter.clientHeight - 5) + 'px';
 
-	oFooter = null;
-	oContent = null;
-	oHeader = null;
+  oFooter = null;
+  oContent = null;
+  oHeader = null;
 }
 
 var scrolling = true;
@@ -546,7 +530,12 @@ function scrollWheel(event){
 // TODO: The sidebar cannot longer be embedded. We can use relative
 // links again and do not need to know the base url any longer :-)
 
+// The refresh snapins do reload after a defined amount of time
 refresh_snapins = null;
+// The restart snapins are notified about the restart of the nagios instance(s)
+restart_snapins = null;
+// Contains a timestamp which holds the time of the last nagios restart handling
+sidebar_restart_time = null;
 
 // Removes a snapin from the sidebar without reloading anything
 function removeSnapin(id, code) {
@@ -564,44 +553,6 @@ function removeSnapin(id, code) {
   myparent = null;
 }
 
-// Updates the contents of a snapin container after get_url
-function updateContents(id, code) {
-  var obj = document.getElementById(id);
-  if (obj) {
-    obj.innerHTML = code;
-    executeJS(id);
-    obj = null;
-  }
-}
-
-// There may be some javascript code in the html code rendered by
-// sidebar.py. Execute it here. This is needed in some browsers.
-function executeJS(objId) {
-  // Before switching to asynchronous requests this worked in firefox
-  // out of the box. Now it seems not to work with ff too. So now
-  // executing the javascript manually.
-  // if (!isFirefox()) {
-  var obj = document.getElementById(objId);
-  var aScripts = obj.getElementsByTagName('script');
-  for(var i in aScripts) {
-    if (aScripts[i].src && aScripts[i].src !== '') {
-      var oScr = document.createElement('script');
-      oScr.src = aScripts[i].src;
-      document.getElementsByTagName("HEAD")[0].appendChild(oScr);
-      oScr = null;
-    } else {
-      try {
-    	  eval(aScripts[i].text);
-      } catch(e) {alert(aScripts[i].text + "\nError:" + e.message);}
-    }
-  }
-  aScripts = null;
-  obj = null;
-}
-
-function isFirefox() {
-  return navigator.userAgent.indexOf("Firefox") > -1;
-}
 
 function toggle_sidebar_snapin(oH2, url) {
     var childs = oH2.parentNode.parentNode.childNodes;
@@ -645,9 +596,34 @@ function switch_site(switchvar) {
        everything is affected by the switch */
 }
 
+/*
+ * Is called as response handler of the generic nagios process restarted
+ * check. If an instance start time is newer than the current stored time
+ * the function triggers a reload of all snapins which have been registered
+ * as restart snapins.
+ */
+function handle_nagios_restarted(_unused, response) {
+    var t = parseFloat(response);
+    if(sidebar_restart_time == t)
+        return;
+    // Some instance has been restarted. Trigger the snapins
+    for (var i in restart_snapins) {
+        var name = restart_snapins[i];
+        get_url("sidebar_snapin.py?name=" + name, updateContents, "snapin_" + name);
+    }
+    sidebar_restart_time = t;
+}
+
 function sidebar_scheduler() {
     var timestamp = Date.parse(new Date()) / 1000;
     var newcontent = "";
+
+    // One generic request to detect if nagios instance(s) have been restarted
+    // For testing hardcoded interval of 30 seconds
+    if(timestamp % 30 == 0 && sidebar_restart_time !== null)
+        get_url('nagios_restarted.py?since=' + escape(sidebar_restart_time),
+                                                       handle_nagios_restarted);
+
     for (var i in refresh_snapins) { 
         var name    = refresh_snapins[i][0];
         var refresh = refresh_snapins[i][1];
@@ -698,19 +674,8 @@ function toggle_folder(o, folderId) {
         }
     }
 
-    if (next) {
-        if (next.style.display == "none") {
-            next.style.display = "";
-            if (img) 
-                img.src = "images/link_folder_open.gif";
-            get_url('customlink_openclose.py?name=' + escape(folderId) + '&state=on');
-        } else {
-            next.style.display = "none";
-            if (img) 
-                img.src = "images/link_folder.gif";
-            get_url('customlink_openclose.py?name=' + escape(folderId) + '&state=off');
-        }
-    }
+    if (next)
+	toggle_tree_state('customlinks', folderId, next);
 
     child = null;
     par = null;
@@ -754,3 +719,38 @@ function initScrollPos() {
 function storeScrollPos() {
     setCookie('sidebarScrollPos', document.getElementById('side_content').scrollTop, null);
 }
+
+/* Foldable Tree in snapin */
+function wato_tree_click(filename) {
+    // Get the URL in the main frame. If that is a view or a dashboard,
+    // then reload that view with an updated "filename" variable. If that is
+    // a wato.py page, navigate to the according wato page. In all other
+    // cases, select the view "allhosts" with the according path.
+    var href = parent.frames[1].location + ""; // adding "" converts url from something magic to string
+    if (href.indexOf("/view.py") >= 0) 
+        href = add_html_var(href, "filename", filename);
+    else if (href.indexOf("/dashboard.py") >= 0) 
+        href = add_html_var(href, "filename", filename);
+    else if (href.indexOf("/wato.py") >= 0)
+        href = "wato.py?filename=" + escape(filename);
+    else
+        href = "view.py?view_name=allhosts&filename=" + escape(filename);
+    parent.frames[1].location = href;
+}
+
+// adds a variable to a GET url, but tries to remove that
+// variable, if it is already existing in the URL. In order
+// to simplify the thing, we just look at the *end* of the
+// URL. This is not perfect but sufficient in order to avoid the URL
+// getting longer and longer.
+function add_html_var(url, varname, value) {
+    var re = new RegExp('&' + varname + '=[^&]*');
+    var new_url = url.replace(re, "");
+    if (new_url.indexOf('?') != '-1')
+        new_url += "&" + varname + "=" + escape(value);
+    else
+        new_url += "?" + varname + "=" + escape(value);
+    return new_url;
+}
+
+
