@@ -25,6 +25,7 @@
 # Boston, MA 02110-1301 USA.
 
 import views, time, defaults
+import weblib
 from lib import *
 
 # Python 2.3 does not have 'set' in normal namespace.
@@ -43,17 +44,17 @@ except NameError:
 #
 # --------------------------------------------------------------
 def render_about():
-    html.write("Version: " + defaults.check_mk_version)
+    html.write(_("Version: ") + defaults.check_mk_version)
     html.write("<ul>")
-    bulletlink("Homepage",        "http://mathias-kettner.de/check_mk.html")
-    bulletlink("Documentation",   "http://mathias-kettner.de/checkmk.html")
-    bulletlink("Download",        "http://mathias-kettner.de/check_mk_download.html")
+    bulletlink(_("Homepage"),        "http://mathias-kettner.de/check_mk.html")
+    bulletlink(_("Documentation"),   "http://mathias-kettner.de/checkmk.html")
+    bulletlink(_("Download"),        "http://mathias-kettner.de/check_mk_download.html")
     bulletlink("Mathias Kettner", "http://mathias-kettner.de")
     html.write("</ul>")
 
 sidebar_snapins["about"] = {
-    "title" : "About Check_MK",
-    "description" : "Version information and Links to Documentation, Homepage and Download of Check_MK",
+    "title" : _("About Check_MK"),
+    "description" : _("Version information and Links to Documentation, Homepage and Download of Check_MK"),
     "author" : "Mathias Kettner",
     "render" : render_about,
     "allowed" : [ "admin", "user", "guest" ],
@@ -69,14 +70,16 @@ sidebar_snapins["about"] = {
 # -----------------------------------------------------------------------
 def render_admin():
     html.write('<ul>')
-    bulletlink("View permissions", "view_permissions.py")
+    if config.may("use_wato"):
+        bulletlink(_("WATO - Configure Hosts and Services"), "wato.py")
+    bulletlink(_("View permissions"), "view_permissions.py")
     if config.may("edit_permissions"):
-        bulletlink("Edit permissions", "edit_permissions.py")
+        bulletlink(_("Edit permissions"), "edit_permissions.py")
     html.write('</ul>')
 
 sidebar_snapins["admin"] = {
-    "title" : "Administration",
-    "description" : "Links to administrations functions, e.g. configuration of permissions",
+    "title" : _("Administration"),
+    "description" : _("Links to administrations functions, e.g. configuration of permissions"),
     "author" : "Mathias Kettner",
     "render" : render_admin,
     "allowed" : [ "admin" ],
@@ -103,37 +106,41 @@ def render_views():
                 continue
             if t == topic:
                 if first:
-                    html.write("<h3>%s</h3>\n" % topic)
+                    html.begin_foldable_container("views", topic, False, topic)
+                    # html.write("<h3>%s</h3>\n" % topic)
                     first = False
-                    html.write("<ul>")
+                    # html.write("<ul>")
                 bulletlink(title, "view.py?view_name=%s" % name)
         if not first: # at least one item rendered
-            html.write("</ul>")
+            html.end_foldable_container()
+            # html.write("</ul>")
 
-    s = [ (view.get("topic", "Other"), view["title"], name) for name, view in html.available_views.items() if not view["hidden"] ]
+    s = [ (view.get("topic", _("Other")), view["title"], name) for name, view in html.available_views.items() if not view["hidden"] ]
     s.sort()
 
     # Enforce a certain order on the topics
-    known_topics = [ "Hosts", "Hostgroups", "Services", "Servicegroups", "Problems", "Addons" ]
+    known_topics = [ "Hosts", "Hostgroups", "Services", "Servicegroups", 
+                     _("Business Intelligence"), _("Problems"), _("Addons") ]
     for topic in known_topics:
         render_topic(topic, s)
 
     rest = list(set([ t for (t, _t, _v) in s if t not in known_topics ]))
     rest.sort()
     for topic in rest:
-        render_topic(topic, s)
+        if topic:
+            render_topic(topic, s)
 
 
     links = []
     if config.may("edit_views"):
         if config.debug:
-            links.append(("EXPORT", "export_views.py"))
-        links.append(("EDIT", "edit_views.py"))
+            links.append((_("EXPORT"), "export_views.py"))
+        links.append((_("EDIT"), "edit_views.py"))
         footnotelinks(links)
 
 sidebar_snapins["views"] = {
-    "title" : "Views",
-    "description" : "Links to all views",
+    "title" : _("Views"),
+    "description" : _("Links to all views"),
     "author" : "Mathias Kettner",
     "render" : render_views,
     "allowed" : [ "user", "admin", "guest" ],
@@ -156,27 +163,29 @@ sidebar_snapins["views"] = {
 def render_groups(what):
     data = html.live.query("GET %sgroups\nColumns: name alias\n" % what)
     name_to_alias = dict(data)
-    groups = [(name_to_alias[name], name) for name in name_to_alias.keys()]
-    groups.sort() # sort by Alias!
+    groups = [(name_to_alias[name].lower(), name_to_alias[name], name) for name in name_to_alias.keys()]
+    groups.sort() # sort by Alias in lowercase
     target = views.get_context_link(html.req.user, "%sgroup" % what)
     if target:
         html.write('<ul>')
-        for alias, name in groups:
+        for alias_lower, alias, name in groups:
             bulletlink(alias, target + "&%sgroup=%s" % (what, htmllib.urlencode(name)))
         html.write('</ul>')
 
 sidebar_snapins["hostgroups"] = {
-    "title" : "Hostgroups",
-    "description" : "Directs links to all host groups",
+    "title" : _("Hostgroups"),
+    "description" : _("Directs links to all host groups"),
     "author" : "Mathias Kettner",
     "render" : lambda: render_groups("host"),
+    "restart":     True,
     "allowed" : [ "user", "admin", "guest" ]
 }
 sidebar_snapins["servicegroups"] = {
-    "title" : "Servicegroups",
-    "description" : "Direct links to all service groups",
+    "title" : _("Servicegroups"),
+    "description" : _("Direct links to all service groups"),
     "author" : "Mathias Kettner",
     "render" : lambda: render_groups("service"),
+    "restart":     True,
     "allowed" : [ "user", "admin", "guest" ]
 }
 
@@ -214,7 +223,6 @@ def render_hosts(mode):
     else:
         num_columns = 2
 
-    views.html = html
     views.load_views()
     target = views.get_context_link(html.req.user, view)
     html.write("<table class=allhosts>\n")
@@ -251,32 +259,35 @@ snapin_allhosts_styles = """
 """
 
 sidebar_snapins["hosts"] = {
-    "title" : "All hosts",
-    "description" : "A summary state of each host with a link to the view showing its services",
+    "title" : _("All hosts"),
+    "description" : _("A summary state of each host with a link to the view showing its services"),
     "author" : "Mathias Kettner",
     "render" : lambda: render_hosts("hosts"),
     "allowed" : [ "user", "admin", "guest" ],
     "refresh" : 60,
+    "restart":     True,
     "styles" : snapin_allhosts_styles,
 }
 
 sidebar_snapins["summary_hosts"] = {
-    "title" : "Summary hosts",
-    "description" : "A summary state of all summary hosts (summary hosts hold aggregated service states and are a feature of Check_MK)",
+    "title" : _("Summary hosts"),
+    "description" : _("A summary state of all summary hosts (summary hosts hold aggregated service states and are a feature of Check_MK)"),
     "author" : "Mathias Kettner",
     "render" : lambda: render_hosts("summary"),
     "allowed" : [ "user", "admin", "guest" ],
     "refresh" : 60,
+    "restart":     True,
     "styles" : snapin_allhosts_styles,
 }
 
 sidebar_snapins["problem_hosts"] = {
-    "title" : "Problem hosts",
-    "description" : "A summary state of all hosts that have problem, with links to problems of those hosts",
+    "title" : _("Problem hosts"),
+    "description" : _("A summary state of all hosts that have problem, with links to problems of those hosts"),
     "author" : "Mathias Kettner",
     "render" : lambda: render_hosts("problems"),
     "allowed" : [ "user", "admin", "guest" ],
     "refresh" : 60,
+    "restart":     True,
     "styles" : snapin_allhosts_styles,
 }
 
@@ -350,8 +361,8 @@ def render_hostmatrix():
 
 
 sidebar_snapins["hostmatrix"] = {
-    "title"       : "Host Matrix",
-    "description" : "A matrix showing s colored square for each host",
+    "title"       : _("Host Matrix"),
+    "description" : _("A matrix showing s colored square for each host"),
     "author"      : "Mathias Kettner",
     "render"      : render_hostmatrix,
     "allowed"     : [ "user", "admin", "guest" ],
@@ -377,15 +388,20 @@ table.hostmatrix td { border: 1px solid white; padding: 0; border-spacing: 0; }
 def render_sitestatus():
     if config.is_multisite():
         html.write("<table cellspacing=0 class=sitestate>")
-        sitenames = config.allsites().keys()
-        sitenames.sort()
-        for sitename in sitenames:
+
+        # Sort the list of sitenames by sitealias
+        sitenames = []
+        for sitename, site in config.allsites().iteritems():
+            sitenames.append((sitename, site['alias']))
+        sitenames = sorted(sitenames, key=lambda k: k[1])
+
+        for sitename, sitealias in sitenames:
             site = config.site(sitename)
             state = html.site_status[sitename]["state"]
             if state == "disabled":
                 switch = "on"
                 text = site["alias"]
-                title = "Site %s is switched off" % site["alias"]
+                title = _("Site %s is switched off") % site["alias"]
             else:
                 switch = "off"
 		try:
@@ -403,30 +419,34 @@ def render_sitestatus():
 
             html.write("<tr><td class=left>%s</td>" % text)
             onclick = "switch_site('_site_switch=%s:%s')" % (sitename, switch)
-            html.write("<td class=\"state %s\">" % state)
-            html.write('<a title="%s" href="#" onclick="%s">%s</a></td>' % (title, onclick, state))
+            html.write("<td class=state>")
+            html.write('<a title="%s" href="#" onclick="%s" class=%s>%s</a></td>' % (title, onclick, state, state))
             html.write("</tr>\n")
         html.write("</table>\n")
 
 
 sidebar_snapins["sitestatus"] = {
-  "title" : "Site status",
-  "description" : "Connection state of each site and button for enabling and disabling the site connection",
+  "title" : _("Site status"),
+  "description" : _("Connection state of each site and button for enabling and disabling the site connection"),
   "author" : "Mathias Kettner",
   "render" : render_sitestatus,
   "allowed" : [ "user", "admin" ],
   "refresh" : 90,
   "styles" : """
-.snapin table.sitestate {
+table.sitestate {
     width: %dpx;
 }
 
-.snapin table.sitestate td {
+table.sitestate td {
     padding: 1px 0px;
     text-align: right;
 }
 
-.snapin table.sitestate td a {
+table.sitestate td.left {
+    text-align: left;
+}
+
+table.sitestate a {
     font-weight: bold;
     -moz-border-radius: 4px;
     margin: 0px;
@@ -434,30 +454,27 @@ sidebar_snapins["sitestatus"] = {
     text-align: center;
     display: block;
 }
-.snapin table.sitestate td.left a {
+table.sitestate td.left a {
     text-align: left;
     font-weight: normal;
 }
 
-.snapin table.sitestate td.state {
+table.sitestate td.state {
     width: 60px;
     font-size: 7pt;
 }
-.snapin table.sitestate td.left {
-    text-align: left;
-}
 
-.snapin table.sitestate td.state a {
+table.sitestate td.state a {
     border-width: 1px;
     border-style: solid;
 }
-.snapin table.sitestate td.online a   { background-color: #3c0; color: #fff; border-color: #0f0; }
-.snapin table.sitestate td.disabled a { background-color: #666; color: #ccc; border-color: #888; }
-.snapin table.sitestate td.dead a     { background-color: #c00; color: #f88; border-color: #f44; }
-.snapin table.sitestate td.waiting a  { background-color: #666; color: #fff; border-color: #ccc; }
-.snapin table.sitestate td.down a     { background-color: #f00; color: #fff; border-color: #800; }
-.snapin table.sitestate td.unreach a  { background-color: #f80; color: #fff; border-color: #840; }
-.snapin table.sitestate td.unknown a  { background-color: #26c; color: #fff; border-color: #44f; }
+table.sitestate a.online   { background-color: #3c0; color: #fff; border-color: #0f0; }
+table.sitestate a.disabled { background-color: #666; color: #ccc; border-color: #888; }
+table.sitestate a.dead     { background-color: #c00; color: #f88; border-color: #f44; }
+table.sitestate a.waiting  { background-color: #666; color: #fff; border-color: #ccc; }
+table.sitestate a.down     { background-color: #f00; color: #fff; border-color: #800; }
+table.sitestate a.unreach  { background-color: #f80; color: #fff; border-color: #840; }
+table.sitestate a.unknown  { background-color: #26c; color: #fff; border-color: #44f; }
 """ % snapin_width
 }
 
@@ -509,31 +526,28 @@ def render_tactical_overview():
         return
     html.write("<table class=\"content_center tacticaloverview\" cellspacing=2 cellpadding=0 border=0>\n")
     for title, data, view, what in [
-            ("Hosts",    hstdata, 'hostproblems', 'host'),
-            ("Services", svcdata, 'svcproblems',  'service'),
+            (_("Hosts"),    hstdata, 'hostproblems', 'host'),
+            (_("Services"), svcdata, 'svcproblems',  'service'),
             ]:
-        html.write("<tr><th>%s</th><th>Problems</th><th>Unhandled</th></tr>\n" % title)
+        html.write("<tr><th>%s</th><th>%s</th><th>%s</th></tr>\n" % \
+                                     (title, _('Problems'), _('Unhandled')))
         html.write("<tr>")
 
         html.write('<td class=total><a target="main" href="view.py?view_name=all%ss">%d</a></td>' % (what, data[0]))
         unhandled = False
         for value in data[1:]:
-            if value > 0:
-                href = "view.py?view_name=" + view
-                if unhandled:
-
-                    href += "&is_%s_acknowledged=0" % what
-                text = link(str(value), href)
-            else:
-                text = str(value)
+            href = "view.py?view_name=" + view
+            if unhandled:
+                href += "&is_%s_acknowledged=0" % what
+            text = link(str(value), href)
             html.write('<td class="%s">%s</td>' % (value == 0 and " " or "states prob", text))
             unhandled = True
         html.write("</tr>\n")
     html.write("</table>\n")
 
 sidebar_snapins["tactical_overview"] = {
-    "title" : "Tactical Overview",
-    "description" : "The total number of hosts and service with and without problems",
+    "title" : _("Tactical Overview"),
+    "description" : _("The total number of hosts and service with and without problems"),
     "author" : "Mathias Kettner",
     "refresh" : 10,
     "render" : render_tactical_overview,
@@ -551,7 +565,7 @@ table.tacticaloverview {
 }
 table.tacticaloverview th { font-size: 7pt; text-align: left; font-weight: normal; padding: 0; padding-top: 2px; }
 table.tacticaloverview td { text-align: right; border: 1px solid #444; padding: 0px; }
-table.tacticaloverview td a { display: block; margin-right: 2px; }
+table.tacticaloverview a { display: block; margin-right: 2px; }
 """ % snapin_width
 }
 # table.tacticaloverview td.prob { font-weight: bold; }
@@ -579,13 +593,14 @@ def render_performance():
     data = html.live.query("GET status\nColumns: external_command_buffer_slots external_command_buffer_max\n")
     size = sum([row[0] for row in data])
     maxx = sum([row[1] for row in data])
-    html.write("<tr><td class=left>Com. buf. max/total</td>"
-               "<td class=right><strong>%d / %d</strong></td></tr>" % (maxx, size))
+    html.write("<tr><td class=left>%s</td>"
+               "<td class=right><strong>%d / %d</strong></td></tr>" % \
+                                        (_('Com. buf. max/total'), maxx, size))
     html.write("</table>\n")
 
 sidebar_snapins["performance"] = {
-    "title" : "Server performance",
-    "description" : "Live monitor of the overall performance of all monitoring servers",
+    "title" : _("Server performance"),
+    "description" : _("Live monitor of the overall performance of all monitoring servers"),
     "author" : "Mathias Kettner",
     "refresh" : 10,
     "render" : render_performance,
@@ -620,8 +635,8 @@ def render_current_time():
     html.write("<div class=time>%s</div>" % time.strftime("%H:%M"))
 
 sidebar_snapins["time"] = {
-    "title" : "Server time",
-    "description" : "A large clock showing the current time of the web server",
+    "title" : _("Server time"),
+    "description" : _("A large clock showing the current time of the web server"),
     "author" : "Mathias Kettner",
     "refresh" : 30,
     "render" : render_current_time,
@@ -703,8 +718,8 @@ def render_nagios():
                 nagioscgilink(text, ref)
 
 sidebar_snapins["nagios_legacy"] = {
-    "title" : "Nagios",
-    "description" : "The classical sidebar of Nagios 3.2.0 with links to your local Nagios instance (no multi site support)",
+    "title" : _("Nagios"),
+    "description" : _("The classical sidebar of Nagios 3.2.0 with links to your local Nagios instance (no multi site support)"),
     "author" : "Mathias Kettner",
     "render" : render_nagios,
     "allowed" : [ "user", "admin", "guest", ],
@@ -720,11 +735,11 @@ sidebar_snapins["nagios_legacy"] = {
 # ----------------------------------------------------------------
 def render_master_control():
     items = [
-        ( "enable_notifications",     "Notifications", ),
-        ( "execute_service_checks",   "Service checks" ),
-        ( "execute_host_checks",      "Host checks" ),
-        ( "enable_event_handlers",    "Event handlers" ),
-        ( "process_performance_data", "Performance data"),
+        ( "enable_notifications",     _("Notifications" )),
+        ( "execute_service_checks",   _("Service checks" )),
+        ( "execute_host_checks",      _("Host checks" )),
+        ( "enable_event_handlers",    _("Event handlers" )),
+        ( "process_performance_data", _("Performance data" )),
         ]
 
     html.live.set_prepend_site(True)
@@ -747,8 +762,8 @@ def render_master_control():
     html.write("</table>")
 
 sidebar_snapins["master_control"] = {
-    "title" : "Master control",
-    "description" : "Buttons for switching globally states such as enabling checks and notifications",
+    "title" : _("Master control"),
+    "description" : _("Buttons for switching globally states such as enabling checks and notifications"),
     "author" : "Mathias Kettner",
     "render" : render_master_control,
     "allowed" : [ "admin", ],
@@ -798,9 +813,7 @@ div#check_mk_sidebar table.master_control td.disabled a {
 """ % snapin_width
 }
 
-def ajax_switch_masterstate(h):
-    global html
-    html = h
+def ajax_switch_masterstate():
     site = html.var("site")
     column = html.var("switch")
     state = int(html.var("state"))
@@ -826,7 +839,7 @@ def ajax_switch_masterstate(h):
         html.live.set_only_sites()
         render_master_control()
     else:
-        html.write("Command %s/%d not found" % (column, state))
+        html.write(_("Command %s/%d not found") % (column, state))
 
 # ---------------------------------------------------------
 #   ____              _                         _
@@ -851,22 +864,20 @@ def render_bookmarks():
     n = 0
     for title, href in bookmarks:
         html.write("<div id=\"bookmark_%d\">" % n)
-        iconbutton("delete", "del_bookmark.py?num=%d" % n, "side", "updateContents", 'snapin_bookmarks')
-        iconbutton("edit", "edit_bookmark.py?num=%d" % n, "main")
+        iconbutton(_("delete"), "del_bookmark.py?num=%d" % n, "side", "updateContents", 'snapin_bookmarks')
+        iconbutton(_("edit"), "edit_bookmark.py?num=%d" % n, "main")
         html.write(link(title, href))
         html.write("</div>")
         n += 1
 
-    html.write("<div class=footnotelink><a href=\"#\" onclick=\"addBookmark()\">Add Bookmark</a></div>\n")
+    html.write("<div class=footnotelink><a href=\"#\" onclick=\"addBookmark()\">%s</a></div>\n" % _('Add Bookmark'))
 
-def page_edit_bookmark(h):
-    global html
-    html = h
-    html.header("Edit Bookmark")
+def page_edit_bookmark():
+    html.header(_("Edit Bookmark"))
     n = int(html.var("num"))
     bookmarks = load_bookmarks()
     if n >= len(bookmarks):
-        raise MKGeneralException("Unknown bookmark id: %d. This is probably a problem with reload or browser history. Please try again." % n)
+        raise MKGeneralException(_("Unknown bookmark id: %d. This is probably a problem with reload or browser history. Please try again.") % n)
 
     if html.var("save") and html.check_transaction():
         title = html.var("title")
@@ -888,41 +899,47 @@ def page_edit_bookmark(h):
         html.set_var("url", url)
 
     html.write("<table class=edit_bookmarks>")
-    html.write("<tr><td>Title:</td><td>")
+    html.write("<tr><td>%s</td><td>" % _('Title:'))
     html.text_input("title", size = 50)
-    html.write("</td></tr><tr><td>URL:</td><td>")
+    html.write("</td></tr><tr><td>%s:</td><td>" % _('URL:'))
     html.text_input("url", size = 50)
     html.write("</td></tr><tr><td></td><td>")
-    html.button("save", "Save")
+    html.button("save", _("Save"))
     html.write("</td></tr></table>\n")
     html.hidden_field("num", str(n))
     html.end_form()
 
     html.footer()
 
-def ajax_del_bookmark(h):
-    global html
-    html = h
+def ajax_del_bookmark():
     num = int(html.var("num"))
     bookmarks = load_bookmarks()
     del bookmarks[num]
     save_bookmarks(bookmarks)
     render_bookmarks()
 
-def ajax_add_bookmark(h):
-    global html
-    html = h
+def ajax_add_bookmark():
     title = html.var("title")
     href = html.var("href")
     if title and href:
         bookmarks = load_bookmarks()
+        # We try to remove http://hostname/some/path/check_mk from the
+        # URI. That keeps the configuration files (bookmarks) portable.
+        # Problem here: We have not access to our own URL, only to the
+        # path part. The trick: we use the Referrer-field from our
+        # request. That points to the sidebar.
+        referer = html.req.headers_in.get("Referer")
+        if referer:
+            while '/' in referer and referer.split('/')[0] == href.split('/')[0]:
+                referer = referer.split('/', 1)[1]
+                href = href.split('/', 1)[1]
         bookmarks.append((title, href))
         save_bookmarks(bookmarks)
     render_bookmarks()
 
 sidebar_snapins["bookmarks"] = {
-    "title" : "Bookmarks",
-    "description" : "A simple and yet practical snapin allowing to create bookmarks to views and other content in the main frame",
+    "title" : _("Bookmarks"),
+    "description" : _("A simple and yet practical snapin allowing to create bookmarks to views and other content in the main frame"),
     "author" : "Mathias Kettner",
     "render" : render_bookmarks,
     "allowed": [ "user", "admin", "guest" ],
@@ -939,28 +956,15 @@ sidebar_snapins["bookmarks"] = {
 #
 # ------------------------------------------------------------
 
-def load_customlink_states():
-    return config.load_user_file("customlinks", {})
-
-def save_customlink_states(states):
-    config.save_user_file("customlinks", states)
-
-def ajax_customlink_openclose(h):
-    global html
-    html = h
-
-    states = load_customlink_states()
-    states[html.var("name")] = html.var("state")
-    save_customlink_states(states)
-
 def render_custom_links():
     links = config.custom_links.get(config.role)
     if not links:
-        html.write("Please edit <tt>%s</tt> in order to configure which links are shown in this snapin.\n" %
-                  (defaults.default_config_dir + "/multisite.mk"))
+        html.write((_("Please edit <tt>%s</tt> in order to configure which links are shown in this snapin.") %
+                  (defaults.default_config_dir + "/multisite.mk")) + "\n")
+        return
 
     def render_list(ids, links):
-        states = load_customlink_states()
+        states = weblib.get_tree_states('customlinks')
         n = 0
         for entry in links:
             n += 1
@@ -988,23 +992,23 @@ def render_custom_links():
                         html.write('<img src="images/link_link.gif">')
                     simplelink(entry[0], entry[1])
                 else:
-                    html.write("Second part of tuple must be list or string, not %s\n" % str(entry[1]))
+                    html.write(_("Second part of tuple must be list or string, not %s\n") % str(entry[1]))
             except Exception, e:
-                html.write("invalid entry %s: %s<br>\n" % (entry, e))
+                html.write(_("invalid entry %s: %s<br>\n") % (entry, e))
 
     render_list([], links)
 
 sidebar_snapins["custom_links"] = {
-    "title" : "Custom Links",
-    "description" : "This snapin contains custom links which can be configured via the configuration variable <tt>custom_links</tt> in <tt>multisite.mk</tt>",
+    "title" : _("Custom Links"),
+    "description" : _("This snapin contains custom links which can be configured via the configuration variable <tt>custom_links</tt> in <tt>multisite.mk</tt>"),
     "author" : "Mathias Kettner",
     "render" : render_custom_links,
     "allowed" : [ "user", "admin", "guest" ],
     "styles" : """
-div#snapin_custom_links div.sublist {
+#snapin_custom_links div.sublist {
     padding-left: 10px;
 }
-div#snapin_custom_links img {
+#snapin_custom_links img {
     margin-right: 5px;
 }
 """
