@@ -42,7 +42,7 @@ class FilterText(Filter):
         htmlvar = self.htmlvars[0]
         current_value = html.var(htmlvar)
         if current_value:
-            return "Filter: %s %s %s\n" % (self.column, self.op, current_value)
+            return _("Filter: %s %s %s\n") % (self.column, self.op, current_value)
         else:
             return ""
 
@@ -50,29 +50,28 @@ class FilterText(Filter):
         return [ (self.htmlvars[0], row[self.column]) ]
 
     def heading_info(self, infoname):
-        htmlvar = self.htmlvars[0]
         return html.var(self.htmlvars[0])
 
 #                               filter          title              info       column           htmlvar
-declare_filter(100, FilterText("hostregex",    "Hostname",        "host",    "host_name",      "host",    "~~"),
-                          "Search field allowing regular expressions and partial matches")
+declare_filter(100, FilterText("hostregex",    _("Hostname"),        "host",    "host_name",      "host",    "~~"),
+                          _("Search field allowing regular expressions and partial matches"))
 
-declare_filter(101, FilterText("host",    "Hostname",             "host",    "host_name",          "host",    "="),
-                          "Exact match, used for linking")
+declare_filter(101, FilterText("host",    _("Hostname"),             "host",    "host_name",          "host",    "="),
+                          _("Exact match, used for linking"))
 
-declare_filter(200, FilterText("serviceregex", "Service",         "service", "service_description",   "service", "~~"),
-                          "Search field allowing regular expressions and partial matches")
+declare_filter(200, FilterText("serviceregex", _("Service"),         "service", "service_description",   "service", "~~"),
+                          _("Search field allowing regular expressions and partial matches"))
 
-declare_filter(201, FilterText("service", "Service",              "service", "service_description",   "service", "="),
-                          "Exact match, used for linking")
+declare_filter(201, FilterText("service", _("Service"),              "service", "service_description",   "service", "="),
+                          _("Exact match, used for linking"))
 
-declare_filter(100, FilterText("servicegroupname", "Servicegroup",   "servicegroup", "servicegroup_name",   "servicegroup_name", "="),
-                          "Exact match, used for linking")
+declare_filter(100, FilterText("servicegroupname", _("Servicegroup"),   "servicegroup", "servicegroup_name",   "servicegroup_name", "="),
+                          _("Exact match, used for linking"))
 
-declare_filter(101, FilterText("servicegroupnameregex", "Servicegroup",   "servicegroup", "servicegroup_name",   "servicegroup_name", "~~"),
-                          "Search field allowing regular expression and partial matches")
+declare_filter(101, FilterText("servicegroupnameregex", _("Servicegroup"),   "servicegroup", "servicegroup_name",   "servicegroup_name", "~~"),
+                          _("Search field allowing regular expression and partial matches"))
 
-declare_filter(202, FilterText("output",  "Service check output", "service", "service_plugin_output", "service_output", "~~"))
+declare_filter(202, FilterText("output",  _("Service check output"), "service", "service_plugin_output", "service_output", "~~"))
 
 
 
@@ -91,21 +90,21 @@ class FilterGroupCombo(Filter):
         if not enforce:
             htmlvars.append("neg_" + htmlvars[0])
         Filter.__init__(self, self.prefix + what + "group", # name,     e.g. "hostgroup"
-                what[0].upper() + what[1:] + "group",       # title,    e.g. "Hostgroup"
-                what,                                       # info,     e.g. "host"
+                what[0].upper() + what[1:].replace("_"," ") + " group",       # title,    e.g. "Hostgroup"
+                what.split("_")[0],                         # info,     e.g. "host"
                 htmlvars,                                   # htmlvars, e.g. "hostgroup"
                 [ what + "group_name" ])                    # rows needed to fetch for link information
         self.what = what
 
     def display(self):
-        choices = all_groups(self.what)
+        choices = all_groups(self.what.split("_")[-1])
         if not self.enforce:
             choices = [("", "")] + choices
         html.select(self.htmlvars[0], choices)
         if not self.enforce:
             html.write(" <nobr>")
             html.checkbox(self.htmlvars[1])
-            html.write("negate")
+            html.write(_("negate"))
             html.write("</nobr>")
 
     def current_value(self, infoname):
@@ -123,10 +122,7 @@ class FilterGroupCombo(Filter):
         if current_value == None:
             return "" # no {what}group exists!
 
-        if self.what + "s" == infoname:
-            col = "groups"
-        else:
-            col = self.what + "_groups"
+        col = self.what + "_groups"
         if not self.enforce and html.var(self.htmlvars[1]):
             negate = "!"
         else:
@@ -154,10 +150,12 @@ class FilterGroupCombo(Filter):
             return alias
 
 
-declare_filter(104, FilterGroupCombo("host",    True),  "Dropdown list, selection of host group is <b>enforced</b>")
-declare_filter(204, FilterGroupCombo("service", True),  "Dropdown list, selection of service group is <b>enforced</b>")
-declare_filter(105, FilterGroupCombo("host",    False), "Optional selection of host group")
-declare_filter(205, FilterGroupCombo("service", False), "Optional selection of service group")
+declare_filter(104, FilterGroupCombo("host",            True),  _("Dropdown list, selection of host group is <b>enforced</b>"))
+declare_filter(204, FilterGroupCombo("service",         True),  _("Dropdown list, selection of service group is <b>enforced</b>"))
+declare_filter(105, FilterGroupCombo("host",            False), _("Optional selection of host group"))
+declare_filter(205, FilterGroupCombo("service",         False), _("Optional selection of service group"))
+declare_filter(106, FilterGroupCombo("host_contact",    False), _("Optional selection of host contact group group"))
+declare_filter(206, FilterGroupCombo("service_contact", False), _("Optional selection of service contact group group"))
 # Livestatus still misses "contact_groups" column.
 # declare_filter(FilterGroupCombo("contact"))
 
@@ -178,9 +176,9 @@ class FilterQueryDropdown(Filter):
         else:
             return ""
 
-declare_filter(110, FilterQueryDropdown("host_check_command", "Host check command", "host", \
+declare_filter(110, FilterQueryDropdown("host_check_command", _("Host check command"), "host", \
         "GET commands\nColumns: name\n", "Filter: host_check_command = %s\n"))
-declare_filter(210, FilterQueryDropdown("check_command", "Service check command", "service", \
+declare_filter(210, FilterQueryDropdown("check_command", _("Service check command"), "service", \
         "GET commands\nColumns: name\n", "Filter: service_check_command = %s\n"))
 
 class FilterServiceState(Filter):
@@ -208,7 +206,7 @@ class FilterServiceState(Filter):
 
         for i in [0,1,2,3]:
             if html.var(self.prefix + "st%d" % i, defval) == "on":
-                if self.prefix == "h":
+                if self.prefix == "hd":
                     column = "service_last_hard_state"
                 else:
                     column = "service_state"
@@ -216,16 +214,18 @@ class FilterServiceState(Filter):
         if html.var(self.prefix + "stp", defval) == "on":
             headers.append("Filter: service_has_been_checked = 0\n")
         if len(headers) == 0:
-            return "Limit: 0\n" # not allowed state
+            return "Limit: 0\n" # no allowed state
+        elif len(headers) == 5: # all states allowed
+            return ""
         else:
             return "".join(headers) + ("Or: %d\n" % len(headers))
 
-declare_filter(215, FilterServiceState("svcstate",     "Service states",      ""))
-declare_filter(216, FilterServiceState("svchardstate", "Service hard states", "h"))
+declare_filter(215, FilterServiceState("svcstate",     _("Service states"),      ""))
+declare_filter(216, FilterServiceState("svchardstate", _("Service hard states"), "hd"))
 
 class FilterHostState(Filter):
     def __init__(self):
-        Filter.__init__(self, "hoststate", "Host states",
+        Filter.__init__(self, "hoststate", _("Host states"),
                 "host", [ "hst0", "hst1", "hst2", "hstp" ], [])
 
     def display(self):
@@ -233,7 +233,7 @@ class FilterHostState(Filter):
             defval = ""
         else:
             defval = "on"
-        for var, text in [("hst0", "UP"), ("hst1", "DOWN"), ("hst2", "UNREACH"), ("hstp", "PENDING")]:
+        for var, text in [("hst0", _("UP")), ("hst1", _("DOWN")), ("hst2", _("UNREACH")), ("hstp", _("PENDING"))]:
             html.checkbox(var, defval)
             html.write(" %s " % text)
 
@@ -265,9 +265,9 @@ class FilterTristate(Filter):
 
     def display(self):
         current = html.var(self.varname)
-        for value, text in [("1", "yes"), ("0", "no"), ("-1", "(ignore)")]:
+        for value, text in [("1", _("yes")), ("0", _("no")), ("-1", _("(ignore)"))]:
             checked = current == value or (current in [ None, ""] and int(value) == self.deflt)
-            html.radiobutton(self.varname, value, checked, text)
+            html.radiobutton(self.varname, value, checked, text + " &nbsp; ")
 
     def tristate_value(self):
         current = html.var(self.varname)
@@ -303,34 +303,34 @@ class FilterNagiosExpression(FilterTristate):
     def filter_code(self, infoname, positive):
         return positive and self.pos or self.neg
 
-declare_filter(120, FilterNagiosExpression("host", "summary_host", "Is summary host",
+declare_filter(120, FilterNagiosExpression("host", "summary_host", _("Is summary host"),
             "Filter: host_custom_variable_names >= _REALNAME\n",
             "Filter: host_custom_variable_names < _REALNAME\n"))
 
-declare_filter(250, FilterNagiosFlag("service", "service_process_performance_data", "Processes performance data"))
-declare_filter(251, FilterNagiosExpression("service", "has_performance_data", "Has performance data",
+declare_filter(250, FilterNagiosFlag("service",       "service_process_performance_data", _("Processes performance data")))
+declare_filter(251, FilterNagiosExpression("service", "has_performance_data",             _("Has performance data"),
             "Filter: service_perf_data != \n",
             "Filter: service_perf_data = \n"))
 
-declare_filter(130, FilterNagiosFlag("host",    "host_in_notification_period",   "Host in notif. period"))
-declare_filter(131, FilterNagiosFlag("host",    "host_acknowledged",             "Host problem has been acknowledged"))
-declare_filter(132, FilterNagiosFlag("host",    "host_active_checks_enabled",     "Host active checks enabled"))
-declare_filter(133, FilterNagiosFlag("host",    "host_notifications_enabled",     "Host notifications enabled"))
-declare_filter(230, FilterNagiosFlag("service", "service_acknowledged",             "Problem acknowledged"))
-declare_filter(231, FilterNagiosFlag("service", "service_in_notification_period",   "Service in notif. per."))
-declare_filter(232, FilterNagiosFlag("service", "service_active_checks_enabled",    "Active checks enabled"))
-declare_filter(233, FilterNagiosFlag("service", "service_notifications_enabled",    "Notifications enabled"))
-declare_filter(236, FilterNagiosFlag("service", "service_is_flapping",              "Flapping"))
-declare_filter(231, FilterNagiosFlag("service", "service_scheduled_downtime_depth", "Service in downtime"))
-declare_filter(132, FilterNagiosFlag("host",    "host_scheduled_downtime_depth", "Host in downtime"))
-declare_filter(232, FilterNagiosExpression("service", "in_downtime", "Host/service in downtime",
+declare_filter(130, FilterNagiosFlag("host",    "host_in_notification_period",      _("Host in notif. period")))
+declare_filter(131, FilterNagiosFlag("host",    "host_acknowledged",                _("Host problem has been acknowledged")))
+declare_filter(132, FilterNagiosFlag("host",    "host_active_checks_enabled",       _("Host active checks enabled")))
+declare_filter(133, FilterNagiosFlag("host",    "host_notifications_enabled",       _("Host notifications enabled")))
+declare_filter(230, FilterNagiosFlag("service", "service_acknowledged",             _("Problem acknowledged")))
+declare_filter(231, FilterNagiosFlag("service", "service_in_notification_period",   _("Service in notif. per.")))
+declare_filter(232, FilterNagiosFlag("service", "service_active_checks_enabled",    _("Active checks enabled")))
+declare_filter(233, FilterNagiosFlag("service", "service_notifications_enabled",    _("Notifications enabled")))
+declare_filter(236, FilterNagiosFlag("service", "service_is_flapping",              _("Flapping")))
+declare_filter(231, FilterNagiosFlag("service", "service_scheduled_downtime_depth", _("Service in downtime")))
+declare_filter(132, FilterNagiosFlag("host",    "host_scheduled_downtime_depth",    _("Host in downtime")))
+declare_filter(232, FilterNagiosExpression("service", "in_downtime",                _("Host/service in downtime"),
             "Filter: service_scheduled_downtime_depth > 0\nFilter: host_scheduled_downtime_depth > 0\nOr: 2\n",
             "Filter: service_scheduled_downtime_depth = 0\nFilter: host_scheduled_downtime_depth = 0\nAnd: 2\n"))
 
 
 class FilterSite(Filter):
     def __init__(self, name, enforce):
-        Filter.__init__(self, name, "Site", None, ["site"], [])
+        Filter.__init__(self, name, _("Site"), None, ["site"], [])
         self.enforce = enforce
 
     def display(self):
@@ -340,11 +340,11 @@ class FilterSite(Filter):
         if config.is_multisite():
             site = html.var("site")
             if site:
-                return "Sites: %s\n" % html.var("site", "")
+                return "%s: %s\n" % (_('Sites'), html.var("site", ""))
             elif not self.enforce:
                 return ""
             else:
-                return "Sites:\n" # no site at all
+                return "%s:\n" % _('Sites') # no site at all
         else:
             return ""
 
@@ -357,8 +357,8 @@ class FilterSite(Filter):
     def variable_settings(self, row):
         return [("site", row["site"])]
 
-declare_filter(500, FilterSite("site",    True), "Selection of site is enforced, use this filter for joining")
-declare_filter(501, FilterSite("siteopt", False), "Optional selection of a site")
+declare_filter(500, FilterSite("site",    True),  _("Selection of site is enforced, use this filter for joining"))
+declare_filter(501, FilterSite("siteopt", False), _("Optional selection of a site"))
 
 # Filter for setting time ranges, e.g. on last_state_change and last_check
 # Variante eins:
@@ -371,7 +371,7 @@ class FilterTime(Filter):
     def __init__(self, info, name, title, column):
         self.column = column
         self.name = name
-        self.ranges = [ (1, "sec"), (60, "min"), (3600, "hours"), (86400, "days") ]
+        self.ranges = [ (1, _("sec")), (60, _("min")), (3600, _("hours")), (86400, _("days")) ]
         Filter.__init__(self, name, title, info, [ name ] + [ name + "_" + n for (s, n) in self.ranges], [column])
 
     def display(self):
@@ -380,13 +380,29 @@ class FilterTime(Filter):
             html.write("<nobr>")
             html.number_input(htmlvar, 0, 2)
             html.write(" %s</nobr> " % n)
-        html.write("<br>\n")
+        if config.filter_columns > 1:
+            html.write("<br>\n")
         current = html.var(self.name, "since")
         for t in [ "before", "since" ]:
-            html.radiobutton(self.name, t, current == t, t)
+            html.radiobutton(self.name, t, current == t, t + " &nbsp; ")
             html.write(" ")
 
     def filter(self, infoname):
+        range = self.get_time_range()
+        if range == None:
+            return ""
+
+        timestamp, negate = range
+        if negate:
+            neg = "!"
+        else:
+            neg = ""
+
+        return "Filter: %s %s>= %d\n" % (self.column, neg, timestamp)
+
+
+    # Extract timerange user has selected from HTML variables
+    def get_time_range(self):
         secs = 0
         for s, n in self.ranges:
             htmlvar = self.name + "_" + n
@@ -399,13 +415,10 @@ class FilterTime(Filter):
 
         if secs > 0:
             timestamp = int(time.time()) - secs
-            if html.var(self.name, "since") != "since":
-                neg = "!"
-            else:
-                neg = ""
-            return "Filter: %s %s>= %d\n" % (self.column, neg, timestamp)
+            neg = html.var(self.name, "since") != "since"
+            return timestamp, neg
         else:
-            return ""
+            return None
 
     # I'm not sure if this function is useful or ever been called.
     # Problem is, that it is not clear wether to use "since" or "before"
@@ -420,10 +433,10 @@ class FilterTime(Filter):
         return vars
 
     def heading_info(self, infoname):
-        return "since the last couple of seconds"
+        return _("since the last couple of seconds")
 
-declare_filter(250, FilterTime("service", "svc_last_state_change", "Last service state change", "service_last_state_change"))
-declare_filter(251, FilterTime("service", "svc_last_check", "Last service check", "service_last_check"))
+declare_filter(250, FilterTime("service", "svc_last_state_change", _("Last service state change"), "service_last_state_change"))
+declare_filter(251, FilterTime("service", "svc_last_check", _("Last service check"), "service_last_check"))
 
 #    _
 #   | |    ___   __ _
@@ -432,7 +445,7 @@ declare_filter(251, FilterTime("service", "svc_last_check", "Last service check"
 #   |_____\___/ \__, |
 #               |___/
 
-declare_filter(252, FilterTime("log", "logtime", "Time of log entry", "log_time"))
+declare_filter(252, FilterTime("log", "logtime", _("Time of log entry"), "log_time"))
 # INFO          0 // all messages not in any other class
 # ALERT         1 // alerts: the change service/host state
 # PROGRAM       2 // important programm events (restart, ...)
@@ -444,11 +457,11 @@ declare_filter(252, FilterTime("log", "logtime", "Time of log entry", "log_time"
 class FilterLogClass(Filter):
     def __init__(self):
         self.log_classes = [
-            (0, "Informational"), (1, "Alerts"), (2, "Program"),
-            (3, "Notifications"), (4, "Passive checks"),
-            (5, "Commands"), (6, "States") ]
+            (0, _("Informational")), (1, _("Alerts")), (2, _("Program")),
+            (3, _("Notifications")), (4, _("Passive checks")),
+            (5, _("Commands")),      (6, _("States")) ]
 
-        Filter.__init__(self, "log_class", "Logentry class",
+        Filter.__init__(self, "log_class", _("Logentry class"),
                 "log", [ "logclass%d" % l for l, c in self.log_classes ], [])
 
     def display(self):
@@ -457,6 +470,10 @@ class FilterLogClass(Filter):
         else:
             defval = "on"
         html.write("<table cellspacing=0 cellpadding=0>")
+        if config.filter_columns == 1:
+            num_cols = 4
+        else:
+            num_cols = 2
         col = 1
         for l, c in self.log_classes:
             if col == 1:
@@ -465,12 +482,12 @@ class FilterLogClass(Filter):
             html.checkbox("logclass%d" % l, defval)
             html.write(c)
             html.write("</td>")
-            if col == 2:
+            if col == num_cols:
                 html.write("</tr>\n")
                 col = 1
             else:
                 col += 1
-        if col == 1:
+        if col < num_cols:
             html.write("<td></td></tr>")
         html.write("</table>\n")
 
@@ -490,4 +507,6 @@ class FilterLogClass(Filter):
             return "".join(headers) + ("Or: %d\n" % len(headers))
 
 declare_filter(255, FilterLogClass())
-declare_filter(202, FilterText("log_plugin_output",  "Log: plugin output", "log", "log_plugin_output", "log_plugin_output", "~~"))
+declare_filter(202, FilterText("log_plugin_output",  _("Log: plugin output"), "log", "log_plugin_output", "log_plugin_output", "~~"))
+declare_filter(260, FilterText("log_contact_name",   _("Log: contact name"),  "log", "log_contact_name",  "log_contact_name",  "="),
+                                                                                                  _("Exact match, used for linking"))
